@@ -1,19 +1,7 @@
 /*
- *  Copyright (c) 2018 Michael Zhou <simeirxh@gmail.com>
+ *  SPDX-FileCopyrightText: 2018 Michael Zhou <simeirxh@gmail.com>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include <QHash>
@@ -37,26 +25,26 @@
 #include <KisViewManager.h>
 #include <KisDocument.h>
 #include <KoResourceServer.h>
+#include <KisStorageModel.h>
 #include <KoResourceServerProvider.h>
 #include <KisPaletteModel.h>
 #include <kis_color_button.h>
 
 #include "KisPaletteEditor.h"
+#include <KisGlobalResourcesInterface.h>
+#include <KisResourceUserOperations.h>
 
 struct KisPaletteEditor::PaletteInfo {
     QString name;
     QString filename;
     int columnCount;
-    bool isGlobal;
-    bool isReadOnly;
+    QString storageLocation;
     QHash<QString, KisSwatchGroup> groups;
 };
 
 struct KisPaletteEditor::Private
 {
-    bool isGlobalModified {false};
     bool isNameModified {false};
-    bool isFilenameModified {false};
     bool isColumnCountModified {false};
     QSet<QString> modifiedGroupNames; // key is original group name
     QSet<QString> newGroupNames;
@@ -67,10 +55,11 @@ struct KisPaletteEditor::Private
     QPointer<KisViewManager> view;
     PaletteInfo modified;
     QPointer<KoDialog> query;
-    KoResourceServer<KoColorSet> *rServer;
+    KoResourceServer<KoColorSet> *rServer {0};
 
     QPalette normalPalette;
     QPalette warnPalette;
+
 };
 
 KisPaletteEditor::KisPaletteEditor(QObject *parent)
@@ -98,73 +87,83 @@ void KisPaletteEditor::setView(KisViewManager *view)
     m_d->view = view;
 }
 
-void KisPaletteEditor::addPalette()
+KoColorSetSP KisPaletteEditor::addPalette()
 {
-    if (!m_d->view) { return; }
-    if (!m_d->view->document()) { return; }
-    KoDialog dlg;
-    QFormLayout layout;
-    dlg.mainWidget()->setLayout(&layout);
-    QLabel lbl(i18nc("Label for line edit to set a palette name.","Name"));
-    QLineEdit le(i18nc("Default name for a new palette","New Palette"));
-    layout.addRow(&lbl, &le);
-    if (dlg.exec() != QDialog::Accepted) { return; }
-    KoColorSet *newColorSet = new KoColorSet(newPaletteFileName(false));
-    newColorSet->setPaletteType(KoColorSet::KPL);
-    newColorSet->setIsGlobal(false);
-    newColorSet->setIsEditable(true);
-    newColorSet->setValid(true);
-    newColorSet->setName(le.text());
-    m_d->rServer->addResource(newColorSet);
-    m_d->rServer->removeFromBlacklist(newColorSet);
+    if (!m_d->view) { return 0; }
+    if (!m_d->view->document()) { return 0; }
 
-    uploadPaletteList();
+    KoColorSetSP colorSet(new KoColorSet());
+
+    KoDialog dialog;
+    QFormLayout *layout = new QFormLayout(dialog.mainWidget());
+    QLineEdit *le = new QLineEdit(i18nc("Default name for a new palette","New Palette"));
+    layout->addRow(i18n("New palette name:"), le);
+
+    QString saveLocation = m_d->rServer->saveLocation();
+
+
+    QCheckBox *chkSaveInDocument = new QCheckBox(i18n("Save Palette in the Current Document"));
+    chkSaveInDocument->setChecked(false);
+    layout->addRow(chkSaveInDocument);
+
+    if (dialog.exec() != QDialog::Accepted) { return 0; }
+
+    QString name = le->text();
+    colorSet->setPaletteType(KoColorSet::KPL);
+    colorSet->setValid(true);
+    colorSet->setName(name);
+    colorSet->setFilename(name.split(" ").join("_")+colorSet->defaultFileExtension());
+
+    QString resourceLocation = "";
+    if (chkSaveInDocument->isChecked()) {
+        resourceLocation = m_d->view->document()->linkedResourcesStorageId();
+    }
+
+    if (KisResourceUserOperations::addResourceWithUserInput(m_d->view->mainWindow(), colorSet, resourceLocation)) {
+
+        return colorSet;
+
+    }
+
+    return 0;
 }
 
-void KisPaletteEditor::importPalette()
+KoColorSetSP KisPaletteEditor::importPalette()
 {
-    KoFileDialog dialog(0, KoFileDialog::OpenFile, "Open Palette");
+    KoFileDialog dialog(nullptr, KoFileDialog::OpenFile, "Open Palette");
+    dialog.setCaption(i18n("Import Palette"));
+
     dialog.setDefaultDir(QDir::homePath());
     dialog.setMimeTypeFilters(QStringList() << "krita/x-colorset" << "application/x-gimp-color-palette");
-    QString filename = dialog.filename();
-    if (filename.isEmpty()) { return; }
-    if (duplicateExistsFilename(filename, false)) {
-        QMessageBox message;
-        message.setWindowTitle(i18n("Can't Import Palette"));
-        message.setText(i18n("Can't import palette: there's already imported with the same filename"));
-        message.exec();
-        return;
-    }
-    KoColorSet *colorSet = new KoColorSet(filename);
-    colorSet->load();
-    QString name = filenameFromPath(colorSet->filename());
-    if (duplicateExistsFilename(name, false)) {
-        colorSet->setFilename(newPaletteFileName(false));
-    } else {
-        colorSet->setFilename(name);
-    }
-    colorSet->setIsGlobal(false);
-    m_d->rServer->addResource(colorSet);
-    m_d->rServer->removeFromBlacklist(colorSet);
 
-    uploadPaletteList();
+    QString filename = dialog.filename();
+    if (filename.isEmpty()) {
+        return nullptr;
+    }
+
+    QMessageBox messageBox;
+    messageBox.setText(i18n("Do you want to store this palette in your current image?"));
+    messageBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
+    QString storageLocation = "";
+    if (messageBox.exec() == QMessageBox::Yes) {
+        storageLocation = m_d->view->document()->linkedResourcesStorageId();
+    }
+    KoResourceSP resource = KisResourceUserOperations::importResourceFileWithUserInput(m_d->view->mainWindow(), storageLocation, ResourceType::Palettes, filename);
+
+    KoColorSetSP palette;
+    if (resource) {
+        palette = resource.dynamicCast<KoColorSet>();
+    }
+
+    return palette;
 }
 
-void KisPaletteEditor::removePalette(KoColorSet *cs)
+void KisPaletteEditor::removePalette(KoColorSetSP cs)
 {
     if (!m_d->view) { return; }
     if (!m_d->view->document()) { return; }
-    if (!cs || !cs->isEditable()) {
-        return;
-    }
-
-    if (cs->isGlobal()) {
-        m_d->rServer->removeResourceAndBlacklist(cs);
-        QFile::remove(cs->filename());
-        return;
-    }
+    if (!cs) { return; }
     m_d->rServer->removeResourceFromServer(cs);
-    uploadPaletteList();
 }
 
 int KisPaletteEditor::rowNumberOfGroup(const QString &oriName) const
@@ -204,18 +203,6 @@ void KisPaletteEditor::rename(const QString &newName)
     m_d->modified.name = newName;
 }
 
-void KisPaletteEditor::changeFilename(const QString &newName)
-{
-    if (newName.isEmpty()) { return; }
-    m_d->isFilenameModified = true;
-    m_d->pathsToRemove.insert(m_d->modified.filename);
-    if (m_d->modified.isGlobal) {
-        m_d->modified.filename = m_d->rServer->saveLocation() + newName;
-    } else {
-        m_d->modified.filename = newName;
-    }
-}
-
 void KisPaletteEditor::changeColCount(int newCount)
 {
     m_d->isColumnCountModified = true;
@@ -224,28 +211,24 @@ void KisPaletteEditor::changeColCount(int newCount)
 
 QString KisPaletteEditor::addGroup()
 {
-    KoDialog dlg;
-    m_d->query = &dlg;
+    KoDialog dialog;
+    m_d->query = &dialog;
 
-    QVBoxLayout layout(&dlg);
-    dlg.mainWidget()->setLayout(&layout);
+    QVBoxLayout *layout = new QVBoxLayout(dialog.mainWidget());
 
-    QLabel lblName(i18n("Name"), &dlg);
-    layout.addWidget(&lblName);
-    QLineEdit leName(&dlg);
-    leName.setText(newGroupName());
-    connect(&leName, SIGNAL(textChanged(QString)), SLOT(slotGroupNameChanged(QString)));
-    layout.addWidget(&leName);
-    QLabel lblRowCount(i18n("Row count"), &dlg);
-    layout.addWidget(&lblRowCount);
-    QSpinBox spxRow(&dlg);
-    spxRow.setValue(20);
-    layout.addWidget(&spxRow);
+    layout->addWidget(new QLabel(i18n("New swatch group name:")));
+    QLineEdit *leName = new QLineEdit(newGroupName());
+    connect(leName, SIGNAL(textChanged(QString)), SLOT(slotGroupNameChanged(QString)));
+    layout->addWidget(leName);
+    layout->addWidget(new QLabel(i18n("Rows of swatches in group:")));
+    QSpinBox *spxRow = new QSpinBox();
+    spxRow->setValue(20);
+    layout->addWidget(spxRow);
 
-    if (dlg.exec() != QDialog::Accepted) { return QString(); }
-    if (duplicateExistsGroupName(leName.text())) { return QString(); }
+    if (dialog.exec() != QDialog::Accepted) { return QString(); }
+    if (duplicateExistsGroupName(leName->text())) { return QString(); }
 
-    QString realName = leName.text();
+    QString realName = leName->text();
     QString name = realName;
     if (duplicateExistsOriginalGroupName(name)) {
         name = newGroupName();
@@ -254,23 +237,23 @@ QString KisPaletteEditor::addGroup()
     KisSwatchGroup &newGroup = m_d->modified.groups[name];
     newGroup.setName(realName);
     m_d->newGroupNames.insert(name);
-    newGroup.setRowCount(spxRow.value());
+    newGroup.setRowCount(spxRow->value());
     return realName;
 }
 
 bool KisPaletteEditor::removeGroup(const QString &name)
 {
-    KoDialog window;
-    window.setWindowTitle(i18nc("@title:window", "Removing Group"));
-    QFormLayout editableItems(&window);
-    QCheckBox chkKeep(&window);
-    window.mainWidget()->setLayout(&editableItems);
-    editableItems.addRow(i18nc("Shows up when deleting a swatch group", "Keep the Colors"), &chkKeep);
-    if (window.exec() != KoDialog::Accepted) { return false; }
+    KoDialog dialog;
+    dialog.setWindowTitle(i18nc("@title:dialog", "Removing Swatch Group"));
+    QFormLayout *editableItems = new QFormLayout(dialog.mainWidget());
+    QCheckBox *chkKeep = new QCheckBox();
+
+    editableItems->addRow(i18nc("Shows up when deleting a swatch group", "Keep the Colors"), chkKeep);
+    if (dialog.exec() != KoDialog::Accepted) { return false; }
 
     m_d->modified.groups.remove(name);
     m_d->newGroupNames.remove(name);
-    if (chkKeep.isChecked()) {
+    if (chkKeep->isChecked()) {
         m_d->keepColorGroups.insert(name);
     }
     return true;
@@ -280,27 +263,26 @@ QString KisPaletteEditor::renameGroup(const QString &oldName)
 {
     if (oldName.isEmpty() || oldName == KoColorSet::GLOBAL_GROUP_NAME) { return QString(); }
 
-    KoDialog dlg;
-    m_d->query = &dlg;
+    KoDialog dialog;
+    m_d->query = &dialog;
     m_d->groupBeingRenamed = m_d->modified.groups[oldName].name();
 
-    QFormLayout form(&dlg);
-    dlg.mainWidget()->setLayout(&form);
+    QFormLayout *form = new QFormLayout(dialog.mainWidget());
 
-    QLineEdit leNewName;
-    connect(&leNewName, SIGNAL(textChanged(QString)), SLOT(slotGroupNameChanged(QString)));
-    leNewName.setText(m_d->modified.groups[oldName].name());
+    QLineEdit *leNewName = new QLineEdit();
+    connect(leNewName, SIGNAL(textChanged(QString)), SLOT(slotGroupNameChanged(QString)));
+    leNewName->setText(m_d->modified.groups[oldName].name());
 
-    form.addRow(i18nc("Renaming swatch group", "New name"), &leNewName);
+    form->addRow(i18n("New swatch group name:"), leNewName);
 
-    if (dlg.exec() != KoDialog::Accepted) { return QString(); }
-    if (leNewName.text().isEmpty()) { return QString(); }
-    if (duplicateExistsGroupName(leNewName.text())) { return QString(); }
+    if (dialog.exec() != KoDialog::Accepted) { return QString(); }
+    if (leNewName->text().isEmpty()) { return QString(); }
+    if (duplicateExistsGroupName(leNewName->text())) { return QString(); }
 
-    m_d->modified.groups[oldName].setName(leNewName.text());
+    m_d->modified.groups[oldName].setName(leNewName->text());
     m_d->modifiedGroupNames.insert(oldName);
 
-    return leNewName.text();
+    return leNewName->text();
 }
 
 void KisPaletteEditor::slotGroupNameChanged(const QString &newName)
@@ -326,16 +308,14 @@ void KisPaletteEditor::changeGroupRowCount(const QString &name, int newRowCount)
     m_d->modifiedGroupNames.insert(name);
 }
 
-void KisPaletteEditor::setGlobal(bool isGlobal)
+void KisPaletteEditor::setStorageLocation(QString location)
 {
-    m_d->isGlobalModified = true;
-    m_d->modified.isGlobal = isGlobal;
+   m_d->modified.storageLocation = location;
 }
 
 void KisPaletteEditor::setEntry(const KoColor &color, const QModelIndex &index)
 {
     Q_ASSERT(m_d->model);
-    if (!m_d->model->colorSet()->isEditable()) { return; }
     if (!m_d->view) { return; }
     if (!m_d->view->document()) { return; }
     KisSwatch c = KisSwatch(color);
@@ -346,37 +326,35 @@ void KisPaletteEditor::setEntry(const KoColor &color, const QModelIndex &index)
 
 void KisPaletteEditor::slotSetDocumentModified()
 {
-    m_d->view->document()->setModified(true);
+    if (m_d->modified.storageLocation == m_d->view->document()->linkedResourcesStorageId()) {
+        updatePalette();
+        KisResourceUserOperations::updateResourceWithUserInput(m_d->view->mainWindow(), m_d->model->colorSet());
+        m_d->view->document()->setModified(true);
+    }
+    m_d->model->colorSet()->setDirty(true);
 }
 
 void KisPaletteEditor::removeEntry(const QModelIndex &index)
 {
     Q_ASSERT(m_d->model);
-    if (!m_d->model->colorSet()->isEditable()) { return; }
     if (!m_d->view) { return; }
     if (!m_d->view->document()) { return; }
     if (qvariant_cast<bool>(index.data(KisPaletteModel::IsGroupNameRole))) {
         removeGroup(qvariant_cast<QString>(index.data(KisPaletteModel::GroupNameRole)));
-        updatePalette();
     } else {
         m_d->model->removeEntry(index, false);
     }
-    if (m_d->model->colorSet()->isGlobal()) {
-        m_d->model->colorSet()->save();
-        return;
-    }
+    updatePalette();
 }
 
 void KisPaletteEditor::modifyEntry(const QModelIndex &index)
 {
-    if (!m_d->model->colorSet()->isEditable()) { return; }
     if (!m_d->view) { return; }
     if (!m_d->view->document()) { return; }
 
-    KoDialog dlg;
-    dlg.setCaption(i18nc("@title:window", "Add a Color"));
-    QFormLayout *editableItems = new QFormLayout(&dlg);
-    dlg.mainWidget()->setLayout(editableItems);
+    KoDialog dialog;
+    dialog.setCaption(i18nc("@title:dialog", "Add a new Color Swatch"));
+    QFormLayout *editableItems = new QFormLayout(dialog.mainWidget());
 
     QString groupName = qvariant_cast<QString>(index.data(Qt::DisplayRole));
     if (qvariant_cast<bool>(index.data(KisPaletteModel::IsGroupNameRole))) {
@@ -385,25 +363,25 @@ void KisPaletteEditor::modifyEntry(const QModelIndex &index)
     }
     else {
 
-        QLineEdit *lnIDName = new QLineEdit(&dlg);
-        QLineEdit *lnGroupName = new QLineEdit(&dlg);
-        KisColorButton *bnColor = new KisColorButton(&dlg);
-        QCheckBox *chkSpot = new QCheckBox(&dlg);
+        QLineEdit *lnIDName = new QLineEdit();
+        QLineEdit *lnGroupName = new QLineEdit();
+        KisColorButton *bnColor = new KisColorButton();
+        QCheckBox *chkSpot = new QCheckBox();
         chkSpot->setToolTip(i18nc("@info:tooltip", "A spot color is a color that the printer is able to print without mixing the paints it has available to it. The opposite is called a process color."));
 
         KisSwatch entry = m_d->model->getEntry(index);
 
-        editableItems->addRow(i18n("ID"), lnIDName);
-        editableItems->addRow(i18nc("Name for a swatch group", "Swatch group name"), lnGroupName);
-        editableItems->addRow(i18n("Color"), bnColor);
-        editableItems->addRow(i18n("Spot"), chkSpot);
+        editableItems->addRow(i18n("Swatch ID:"), lnIDName);
+        editableItems->addRow(i18n("Color swatch name:"), lnGroupName);
+        editableItems->addRow(i18nc("Color as the Color of a Swatch in a Palette", "Color:"), bnColor);
+        editableItems->addRow(i18n("Spot color:"), chkSpot);
 
         lnGroupName->setText(entry.name());
         lnIDName->setText(entry.id());
         bnColor->setColor(entry.color());
         chkSpot->setChecked(entry.spotColor());
 
-        if (dlg.exec() == KoDialog::Accepted) {
+        if (dialog.exec() == KoDialog::Accepted) {
             entry.setName(lnGroupName->text());
             entry.setId(lnIDName->text());
             entry.setColor(bnColor->color());
@@ -418,83 +396,79 @@ void KisPaletteEditor::addEntry(const KoColor &color)
     Q_ASSERT(m_d->model);
     if (!m_d->view) { return; }
     if (!m_d->view->document()) { return; }
-    if (!m_d->model->colorSet()->isEditable()) { return; }
-    KoDialog window;
-    window.setWindowTitle(i18nc("@title:window", "Add a new Colorset Entry"));
-    QFormLayout editableItems(&window);
-    window.mainWidget()->setLayout(&editableItems);
-    QComboBox cmbGroups(&window);
-    cmbGroups.addItems(m_d->model->colorSet()->getGroupNames());
-    QLineEdit lnIDName(&window);
-    QLineEdit lnName(&window);
-    KisColorButton bnColor(&window);
-    QCheckBox chkSpot(&window);
-    chkSpot.setToolTip(i18nc("@info:tooltip", "A spot color is a color that the printer is able to print without mixing the paints it has available to it. The opposite is called a process color."));
-    editableItems.addRow(i18n("Group"), &cmbGroups);
-    editableItems.addRow(i18n("ID"), &lnIDName);
-    editableItems.addRow(i18n("Name"), &lnName);
-    editableItems.addRow(i18n("Color"), &bnColor);
-    editableItems.addRow(i18nc("Spot color", "Spot"), &chkSpot);
-    cmbGroups.setCurrentIndex(0);
-    lnName.setText(i18nc("Default name for a color swatch","Color %1", QString::number(m_d->model->colorSet()->colorCount()+1)));
-    lnIDName.setText(QString::number(m_d->model->colorSet()->colorCount() + 1));
-    bnColor.setColor(color);
-    chkSpot.setChecked(false);
 
-    if (window.exec() != KoDialog::Accepted) { return; }
+    KoDialog dialog;
+    dialog.setWindowTitle(i18nc("@title:dialog", "Add a new Color Swatch"));
 
-    QString groupName = cmbGroups.currentText();
+    QFormLayout *editableItems = new QFormLayout(dialog.mainWidget());
+
+    QComboBox *cmbGroups = new QComboBox();
+    cmbGroups->addItems(m_d->model->colorSet()->getGroupNames());
+    cmbGroups->setCurrentIndex(0);
+
+    QLineEdit *lnIDName = new QLineEdit();
+    lnIDName->setText(QString::number(m_d->model->colorSet()->colorCount() + 1));
+
+    QLineEdit *lnName = new QLineEdit();
+    lnName->setText(i18nc("Default name for a color swatch","Color %1", QString::number(m_d->model->colorSet()->colorCount()+1)));
+
+    KisColorButton *bnColor = new KisColorButton();
+    bnColor->setColor(color);
+
+    QCheckBox *chkSpot = new QCheckBox();
+    chkSpot->setChecked(false);
+    chkSpot->setToolTip(i18nc("@info:tooltip", "A spot color is a color that the printer is able to print without mixing the paints it has available to it. The opposite is called a process color."));
+
+    editableItems->addRow(i18n("Swatch Group:"), cmbGroups);
+    editableItems->addRow(i18n("Swatch ID:"), lnIDName);
+    editableItems->addRow(i18n("Color swatch name:"), lnName);
+    editableItems->addRow(i18nc("Color as the Color of a Swatch in a Palette", "Color:"), bnColor);
+    editableItems->addRow(i18n("Spot color:"), chkSpot);
+
+    if (dialog.exec() != KoDialog::Accepted) { return; }
+
+    QString groupName = cmbGroups->currentText();
 
     KisSwatch newEntry;
-    newEntry.setColor(bnColor.color());
-    newEntry.setName(lnName.text());
-    newEntry.setId(lnIDName.text());
-    newEntry.setSpotColor(chkSpot.isChecked());
+    newEntry.setColor(bnColor->color());
+    newEntry.setName(lnName->text());
+    newEntry.setId(lnIDName->text());
+    newEntry.setSpotColor(chkSpot->isChecked());
     m_d->model->addEntry(newEntry, groupName);
-
-    if (m_d->model->colorSet()->isGlobal()) {
-        m_d->model->colorSet()->save();
-        return;
-    }
     m_d->modifiedGroupNames.insert(groupName);
     m_d->modified.groups[groupName].addEntry(newEntry);
 }
 
+bool KisPaletteEditor::isModified() const
+{
+    if (m_d->model->colorSet()) {
+        return m_d->model->colorSet()->isDirty();
+    } else {
+        return false;
+    }
+}
+
 void KisPaletteEditor::updatePalette()
 {
+    dbgResources << Q_FUNC_INFO << "updating the palette model inside the palette editor object";
     Q_ASSERT(m_d->model);
     Q_ASSERT(m_d->model->colorSet());
-    if (!m_d->model->colorSet()->isEditable()) { return; }
     if (!m_d->view) { return; }
     if (!m_d->view->document()) { return; }
-    KoColorSet *palette = m_d->model->colorSet();
+    KoColorSetSP palette = m_d->model->colorSet();
     PaletteInfo &modified = m_d->modified;
 
     if (m_d->isColumnCountModified) {
         palette->setColumnCount(modified.columnCount);
     }
     if (m_d->isNameModified) {
-        palette->setName(modified.name);
+        KisResourceUserOperations::renameResourceWithUserInput(m_d->view->mainWindow(), palette, m_d->modified.name);
     }
-    if (m_d->isFilenameModified) {
-        QString originalPath = palette->filename();
-        palette->setFilename(modified.filename);
-        if (palette->isGlobal()) {
-            if (!palette->save()) {
-                palette->setFilename(newPaletteFileName(true));
-                palette->save();
-            }
-            QFile::remove(originalPath);
-        }
-    }
-    if (m_d->isGlobalModified) {
-        palette->setIsGlobal(modified.isGlobal);
-        if (modified.isGlobal) {
-            setGlobal();
-        } else {
-            setNonGlobal();
-        }
+    QString resourceLocation = m_d->model->colorSet()->storageLocation();
+    if (resourceLocation != m_d->modified.storageLocation) {
+        // We need functionality for moving the resource to the new resource storage...
     }
+
     Q_FOREACH (const QString &groupName, palette->getGroupNames()) {
         if (!modified.groups.contains(groupName)) {
             m_d->model->removeGroup(groupName, m_d->keepColorGroups.contains(groupName));
@@ -516,27 +490,51 @@ void KisPaletteEditor::updatePalette()
         m_d->model->addGroup(modified.groups[newGroupName]);
     }
     m_d->newGroupNames.clear();
+}
 
-    if (m_d->model->colorSet()->isGlobal()) {
-        m_d->model->colorSet()->save();
+void KisPaletteEditor::saveNewPaletteVersion()
+{
+    if (!m_d->model || !m_d->model->colorSet()) { return; }
+
+    QModelIndex index = m_d->rServer->resourceModel()->indexForResource(m_d->model->colorSet());
+    bool isGlobal = false;
+    if (index.isValid()) {
+        bool ok = false;
+        int storageId = m_d->rServer->resourceModel()->data(index, Qt::UserRole + KisAllResourcesModel::StorageId).toInt(&ok);
+        if (ok) {
+            KisStorageModel storageModel;
+            KisResourceStorageSP storage = storageModel.storageForId(storageId);
+            isGlobal = storage->type() != KisResourceStorage::StorageType::Memory;
+        }
     }
+    bool res = false;
+    if (isGlobal) {
+        if (m_d->view) {
+            res = KisResourceUserOperations::updateResourceWithUserInput(m_d->view->mainWindow(), m_d->model->colorSet());
+        } else if (m_d->model->colorSet()->version() >= 0) {
+            //if the version is -1, then the resource should not be updated, because it was never saved to begin with...
+            res = m_d->rServer->resourceModel()->updateResource(m_d->model->colorSet());
+            dbgResources << Q_FUNC_INFO << "-- Updating resource without user input: " << m_d->model->colorSet()->name() << "Result:" << res;
+        }
+    }
+
+    // let's not change it to true if it wasn't modified at all
+    m_d->model->colorSet()->setDirty(m_d->model->colorSet()->isDirty() && !res);
 }
 
 void KisPaletteEditor::slotPaletteChanged()
 {
     Q_ASSERT(m_d->model);
     if (!m_d->model->colorSet()) { return; }
-    KoColorSet *palette = m_d->model->colorSet();
+    KoColorSetSP palette = m_d->model->colorSet();
     m_d->modified.groups.clear();
     m_d->keepColorGroups.clear();
     m_d->newGroupNames.clear();
     m_d->modifiedGroupNames.clear();
 
     m_d->modified.name = palette->name();
-    m_d->modified.filename = palette->filename();
+    m_d->modified.storageLocation = palette->storageLocation();
     m_d->modified.columnCount = palette->columnCount();
-    m_d->modified.isGlobal = palette->isGlobal();
-    m_d->modified.isReadOnly = !palette->isEditable();
 
     Q_FOREACH (const QString &groupName, palette->getGroupNames()) {
         KisSwatchGroup *cs = palette->getGroup(groupName);
@@ -544,95 +542,11 @@ void KisPaletteEditor::slotPaletteChanged()
     }
 }
 
-void KisPaletteEditor::setGlobal()
-{
-    Q_ASSERT(m_d->model);
-    if (!m_d->view) { return; }
-    if (!m_d->view->document()) { return; }
-    if (!m_d->model->colorSet()) { return; }
-
-    KoColorSet *colorSet = m_d->model->colorSet();
-    QString saveLocation = m_d->rServer->saveLocation();
-    QString name = filenameFromPath(colorSet->filename());
-
-    QFileInfo fileInfo(saveLocation + name);
-
-    colorSet->setFilename(fileInfo.filePath());
-    colorSet->setIsGlobal(true);
-    m_d->rServer->removeFromBlacklist(colorSet);
-    if (!colorSet->save()) {
-        QMessageBox message;
-        message.setWindowTitle(i18n("Saving palette failed"));
-        message.setText(i18n("Failed to save global palette file. Please set it to non-global, or you will lose the file when you close Krita"));
-        message.exec();
-    }
-
-    uploadPaletteList();
-}
-
-bool KisPaletteEditor::duplicateExistsFilename(const QString &filename, bool global) const
-{
-    QString prefix;
-    if (global) {
-        prefix = m_d->rServer->saveLocation();
-    }
-
-    Q_FOREACH (const KoResource *r, KoResourceServerProvider::instance()->paletteServer()->resources()) {
-        if (r->filename() == prefix + filename && r != m_d->model->colorSet()) {
-            return true;
-        }
-    }
-
-    return false;
-}
-
 QString KisPaletteEditor::relativePathFromSaveLocation() const
 {
     return filenameFromPath(m_d->modified.filename);
 }
 
-void KisPaletteEditor::setNonGlobal()
-{
-    Q_ASSERT(m_d->model);
-    if (!m_d->view) { return; }
-    if (!m_d->view->document()) { return; }
-    if (!m_d->model->colorSet()) { return; }
-    KoColorSet *colorSet = m_d->model->colorSet();
-    QString name = filenameFromPath(colorSet->filename());
-    QFile::remove(colorSet->filename());
-    if (duplicateExistsFilename(name, false)) {
-        colorSet->setFilename(newPaletteFileName(false));
-    } else {
-        colorSet->setFilename(name);
-    }
-    colorSet->setIsGlobal(false);
-
-    uploadPaletteList();
-}
-
-QString KisPaletteEditor::newPaletteFileName(bool isGlobal)
-{
-    QSet<QString> nameSet;
-
-    Q_FOREACH (const KoResource *r, m_d->rServer->resources()) {
-        nameSet.insert(r->filename());
-    }
-
-    KoColorSet tmpColorSet;
-    QString result = "new_palette_";
-
-    if (isGlobal) {
-        result = m_d->rServer->saveLocation() + result;
-    }
-
-    int i = 0;
-    while (nameSet.contains(result + QString::number(i) + tmpColorSet.defaultFileExtension())) {
-        i++;
-    }
-    result = result + QString::number(i) + tmpColorSet.defaultFileExtension();
-    return result;
-}
-
 QString KisPaletteEditor::newGroupName() const
 {
     int i = 1;
@@ -644,19 +558,6 @@ QString KisPaletteEditor::newGroupName() const
     return groupname;
 }
 
-void KisPaletteEditor::uploadPaletteList() const
-{
-    QList<KoColorSet *> list;
-    Q_FOREACH (KoResource * paletteResource, m_d->rServer->resources()) {
-        KoColorSet *palette = static_cast<KoColorSet*>(paletteResource);
-        Q_ASSERT(palette);
-        if (!palette->isGlobal()) {
-            list.append(palette);
-        }
-    }
-    m_d->view->document()->setPaletteList(list);
-}
-
 QString KisPaletteEditor::filenameFromPath(const QString &path) const
 {
     return QDir::fromNativeSeparators(path).section('/', -1, -1);
