@@ -1,23 +1,11 @@
 /*
- *  Copyright (c) 2002 Patrick Julien <freak@codepimps.org>
- *  Copyright (c) 2004-2008 Boudewijn Rempt <boud@valdyas.org>
- *  Copyright (c) 2004 Clarence Dang <dang@kde.org>
- *  Copyright (c) 2004 Adrian Page <adrian@pagenet.plus.com>
- *  Copyright (c) 2004,2010 Cyrille Berger <cberger@cberger.net>
+ *  SPDX-FileCopyrightText: 2002 Patrick Julien <freak@codepimps.org>
+ *  SPDX-FileCopyrightText: 2004-2008 Boudewijn Rempt <boud@valdyas.org>
+ *  SPDX-FileCopyrightText: 2004 Clarence Dang <dang@kde.org>
+ *  SPDX-FileCopyrightText: 2004 Adrian Page <adrian@pagenet.plus.com>
+ *  SPDX-FileCopyrightText: 2004, 2010 Cyrille Berger <cberger@cberger.net>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "kis_duplicateop.h"
@@ -59,6 +47,7 @@
 #include <kis_random_sub_accessor.h>
 #include <kis_fixed_paint_device.h>
 #include <kis_iterator_ng.h>
+#include <kis_spacing_information.h>
 
 #include "kis_duplicateop_settings.h"
 #include "kis_duplicateop_settings_widget.h"
@@ -75,7 +64,6 @@ KisDuplicateOp::KisDuplicateOp(const KisPaintOpSettingsSP settings, KisPainter *
     m_sizeOption.readOptionSetting(settings);
     m_rotationOption.readOptionSetting(settings);
     m_opacityOption.readOptionSetting(settings);
-
     m_sizeOption.resetAllSensors();
     m_rotationOption.resetAllSensors();
     m_opacityOption.resetAllSensors();
@@ -130,7 +118,9 @@ KisSpacingInformation KisDuplicateOp::paintAt(const KisPaintInformation& info)
     }
 
     qreal rotation = m_rotationOption.apply(info);
+
     qreal opacity = m_opacityOption.apply(painter(),info);
+
     qreal scale = m_sizeOption.apply(info);
 
     if (checkSizeTooSmall(scale)) return KisSpacingInformation();
@@ -193,11 +183,11 @@ KisSpacingInformation KisDuplicateOp::paintAt(const KisPaintInformation& info)
     //     KisRandomSubAccessorSP srcAcc = realSourceDevice->createRandomSubAccessor();
 
     //     //Action
-    //     do {
+    //     while (dstIt.nextPixel()) {
     //         QPointF p =  KisPerspectiveMath::matProd(startM, KisPerspectiveMath::matProd(endM, QPointF(dstIt.x() + dstRect.x(), dstIt.y() + dstRect.y())) + translat);
     //         srcAcc->moveTo(p);
     //         srcAcc->sampledOldRawData(dstIt.rawData());
-    //     } while (dstIt.nextPixel());
+    //     }
 
 
     // }
@@ -217,7 +207,7 @@ KisSpacingInformation KisDuplicateOp::paintAt(const KisPaintInformation& info)
         const bool smallHeight = healRect.height() < 3;
 
         if (smallWidth || smallHeight) {
-            healRect.adjust(-smallWidth, -smallHeight, smallWidth, smallHeight);
+            healRect.adjust(-1, -1, 1, 1);
         }
 
         const int healSW = healRect.width();
@@ -293,3 +283,8 @@ KisSpacingInformation KisDuplicateOp::paintAt(const KisPaintInformation& info)
 
     return effectiveSpacing(scale);
 }
+
+KisSpacingInformation KisDuplicateOp::updateSpacingImpl(const KisPaintInformation &info) const
+{
+    return effectiveSpacing(m_sizeOption.apply(info));
+}
